pub enum ReceiveErrorTimeout {
Closed,
Timeout,
}Expand description
Error type for channel receive operations with timeout
Variants
Closed
Indicates that the channel is closed on both sides with a call to
close()
Timeout
Indicates that channel operation reached timeout and is canceled