pub enum SendTimeoutError<T> {
Closed(T),
Timeout(T),
}Expand description
Error type for channel send operations with timeout
Variants
Closed(T)
Indicates that the channel is closed on both sides with a call to
close()
Timeout(T)
Indicates that channel operation reached timeout and is canceled
Implementations
Sourceimpl<T> SendTimeoutError<T>
impl<T> SendTimeoutError<T>
Sourcepub fn into_inner(self) -> T
pub fn into_inner(self) -> T
Consumes the error and returns the contained value.