pub struct DrainIntoBlockingFuture<'a, 'b, T> { /* private fields */ }Expand description
DrainIntoBlockingFuture is a future for draining all available messages from a channel into a vector, blocking until at least one message is received.