pub struct ReceiveFuture<'a, T> { /* private fields */ }Expand description
ReceiveFuture is a future for receiving an object from a channel asynchronously. It must be polled to complete the receive operation.