pub struct SendManyFuture<'a, 'b, T> { /* private fields */ }Expand description
SendManyFuture is a future for sending multiple objects to a channel asynchronously. It must be polled to complete the send operation.